/* Name: macrokeyboard01.c
   Description:
Version 20140830



Sends macros red from SD card to USB.
Improvements:

Version 20140830:

- added abort button
- added SD CARD error 1s alert if file doesn't exist


Version 20140828:

- Sends no key pressed after USB is plugged in prior to main loop in order to not miss beginning of macro.

- Krsta has added release codes for correct/faster releases

- Added initial re-check of SD card it fails.

- Added stop of blok read/type execution in case of block read error ( reset is necessary ).
PD7 - USB initialized after plugged in
PD6 - Macro is red from SD card
PD5 - SD CARD error
   
 */

//V-USB:

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>

#include "usbdrv.h"
#include "oddebug.h"

//SD:

#include <util/delay.h>
#include "SPI_routines.h"
#include "SD_routines.h"
#include "UART_routines.h"
#include "FAT32.h"



//V-USB:
uchar red_character = 0; //!!! *** character to be "typed" on keyboard (0 means to not type anything, but '0' means to type character 0) *** !!!

//SD:
volatile unsigned long startBlock;
volatile unsigned long totalBlocks;
volatile unsigned char buffer[512];
volatile unsigned long firstDataSector, rootCluster, totalClusters;
volatile unsigned int  bytesPerSector, sectorPerCluster, reservedSectorCount;

/* ----------------------- hardware I/O abstraction ------------------------ */

static void USB_hardwareInit(void)
{
	
//V-USB:
/* pin assignments:
	
PD1	debug tx
PD2	USB+ (int0)
PD3	USB-

*/

uchar	i, j;

    //PORTD = 0b11110011;   /* 1111 1010 bin: activate pull-ups except on USB lines */
	PORTD = (0<<2)|(0<<3);
    //DDRD =  0b00001100;    /* 0000 0111 bin: all pins input except USB (-> USB reset) */
	PORTD = (1<<2)|(1<<3);
	j = 0;
	while(--j){     /* USB Reset by device only required on Watchdog Reset */
		i = 0;
		while(--i); /* delay >10ms for USB reset */
	}
    //DDRD = 0b00000000;    /* 0000 0010 bin: remove USB reset condition */
	PORTD = (0<<2)|(0<<3);
    /* configure timer 0 for a rate of 12M/(1024 * 256) = 45.78 Hz (~22ms) */
    TCCR0 = 5;      /* timer 0 prescaler: 1024 */
}

void SD_port_init(void)
{

//SD:

PORTB = 0b11101111;
DDRB  = 0b10111111; //MISO line i/p, rest o/p

}

//SD:

//UART0 initialize
// desired baud rate: 28800 for 12MHZ
// actual: baud rate:???
// char size: 8 bit
// parity: Disabled

void uart0_init(void)
{
	UCSRB = 0x00; //disable while setting baud rate
	UCSRA = 0x00;
	UCSRC = (1 << URSEL) | 0x06;
	UBRRL = 0x19; //set baud rate lo
	UBRRH = 0x00; //set baud rate hi
	UCSRB = 0x18;
}
//BUTTONS:
void init_buttons(void)
{
	DDRA=0x00;	//all A port pins as input
	PORTA=0xFF;	//Enable internal Pull-Up resistors
	DDRC=0x00;	//all C port pins as input
	PORTC=0xFF;	//Enable internal Pull-Up resistors	
}
//LEDS:
void init_leds(void)
{
		DDRD = (1<<PD7)|(1<<PD6)|(1<<PD5); //set pins as output
		//PD7 = USB ready
		//PD6 = Processing macro
		//PD5 = SD CARD error
}
//SD:
//call this routine to initialize all peripherals
void init_devices(void)
{
	cli();
	SD_port_init();
	spi_init();
	uart0_init();

	init_buttons();
	init_leds();

	MCUCR = 0x00;
	GICR  = 0x00;
	TIMSK = 0x00; //timer interrupt sources
	//all peripherals are now initialized
}



/* ------------------------------------------------------------------------- */
//V-USB:
#define NUM_KEYS    128

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */
//V-USB:
static uchar    reportBuffer[2];    /* buffer for HID reports */
static uchar    idleRate;           /* in 4 ms units */
//V-USB:
const PROGMEM char usbHidReportDescriptor[35] = {   /* USB report descriptor */
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x06,                    // USAGE (Keyboard)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
    0x19, 0xe0,                    //   USAGE_MINIMUM (Keyboard LeftControl)
    0x29, 0xe7,                    //   USAGE_MAXIMUM (Keyboard Right GUI)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x95, 0x08,                    //   REPORT_COUNT (8)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x25, 0x65,                    //   LOGICAL_MAXIMUM (101)
    0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))
    0x29, 0x65,                    //   USAGE_MAXIMUM (Keyboard Application)
    0x81, 0x00,                    //   INPUT (Data,Ary,Abs)
    0xc0                           // END_COLLECTION
};
/* We use a simplifed keyboard report descriptor which does not support the
 * boot protocol. We don't allow setting status LEDs and we only allow one
 * simultaneous key press (except modifiers). We can therefore use short
 * 2 byte input reports.
 * The report descriptor has been created with usb.org's "HID Descriptor Tool"
 * which can be downloaded from http://www.usb.org/developers/hidpage/.
 * Redundant entries (such as LOGICAL_MINIMUM and USAGE_PAGE) have been omitted
 * for the second INPUT item.
 */
//V-USB:
/* Keyboard usage values, see usb.org's HID-usage-tables document, chapter
 * 10 Keyboard/Keypad Page for more codes.
 */
#define MOD_CONTROL_LEFT    (1<<0)
#define MOD_SHIFT_LEFT      (1<<1)
#define MOD_ALT_LEFT        (1<<2)
#define MOD_GUI_LEFT        (1<<3)
#define MOD_CONTROL_RIGHT   (1<<4)
#define MOD_SHIFT_RIGHT     (1<<5)
#define MOD_ALT_RIGHT       (1<<6)
#define MOD_GUI_RIGHT       (1<<7)

#define KEY_        0 //for non-typable characters

#define KEY_A       4
#define KEY_B       5
#define KEY_C       6
#define KEY_D       7
#define KEY_E       8
#define KEY_F       9
#define KEY_G       10
#define KEY_H       11
#define KEY_I       12
#define KEY_J       13
#define KEY_K       14
#define KEY_L       15
#define KEY_M       16
#define KEY_N       17
#define KEY_O       18
#define KEY_P       19
#define KEY_Q       20
#define KEY_R       21
#define KEY_S       22
#define KEY_T       23
#define KEY_U       24
#define KEY_V       25
#define KEY_W       26
#define KEY_X       27
#define KEY_Y       28
#define KEY_Z       29
#define KEY_1       30
#define KEY_2       31
#define KEY_3       32
#define KEY_4       33
#define KEY_5       34
#define KEY_6       35
#define KEY_7       36
#define KEY_8       37
#define KEY_9       38
#define KEY_0       39
#define KEY_ENTER   40
#define KEY_ESC     41
#define KEY_DEL     42
#define KEY_TAB     43
#define KEY_SPACE   44
#define KEY_MINUS   45
#define KEY_EQ      46
#define KEY_BRACKET_L      47
#define KEY_BRACKET_R      48
#define KEY_BACKSLASH      49
#define KEY_NON_US_POUND   50
#define KEY_SEMICOLON   51
#define KEY_APOSTROPHE   52
#define KEY_GRAVE_ACCENT 53
#define KEY_COMMA 54
#define KEY_DOT 55
#define KEY_SLASH 56
#define KEY_CAPSLOCK 57
//Omitting function keys Fx, pgup/down, arrows etc... We need only keys for typing of text...

//V-USB:
/* This will look up the correct character to red character according to the ASCII table's order.
 * Modifier is used for chars requiring SHIFT to be pressed.
 * This wil decide the numbers sent to USB (different from ASCII).
*/
static const uchar  keyReport[NUM_KEYS + 1][2] PROGMEM = {
/* none */  {0, 0},                     /* no key pressed */
/* 1 */    {0, KEY_},
/* 2 */    {0, KEY_},
/* 3 */    {0, KEY_},
/* 4 */    {0, KEY_},
/* 5 */    {0, KEY_},
/* 6 */    {0, KEY_},
/* 7 */    {0, KEY_},
/* 8 */    {0, KEY_},
/* 9 */    {0, KEY_TAB},
/* 10 */    {0, KEY_},
/* 11 */    {0, KEY_},
/* 12 */    {0, KEY_},
/* 13 */    {0, KEY_ENTER},
/* 14 */    {0, KEY_},
/* 15 */    {0, KEY_},
/* 16 */    {0, KEY_},
/* 17 */    {0, KEY_},
/* 18 */    {0, KEY_},
/* 19 */    {0, KEY_},
/* 20 */    {0, KEY_},
/* 21 */    {0, KEY_},
/* 22 */    {0, KEY_},
/* 23 */    {0, KEY_},
/* 24 */    {0, KEY_},
/* 25 */    {0, KEY_},
/* 26 */    {0, KEY_},
/* 27 */    {0, KEY_ESC},
/* 28 */    {0, KEY_},
/* 29 */    {0, KEY_},
/* 30 */    {0, KEY_},
/* 31 */    {0, KEY_},
/* 32 */    {0, KEY_SPACE},
/* 33 */    {MOD_SHIFT_LEFT, KEY_1},
/* 34 */    {MOD_SHIFT_LEFT, KEY_APOSTROPHE},
/* 35 */    {MOD_SHIFT_LEFT, KEY_3},
/* 36 */    {MOD_SHIFT_LEFT, KEY_4},
/* 37 */    {MOD_SHIFT_LEFT, KEY_5},
/* 38 */    {MOD_SHIFT_LEFT, KEY_7},
/* 39 */    {0, KEY_APOSTROPHE},
/* 40 */    {MOD_SHIFT_LEFT, KEY_9},
/* 41 */    {MOD_SHIFT_LEFT, KEY_0},
/* 42 */    {MOD_SHIFT_LEFT, KEY_8},
/* 43 */    {MOD_SHIFT_LEFT, KEY_EQ},
/* 44 */    {0, KEY_COMMA},
/* 45 */    {0, KEY_MINUS},
/* 46 */    {0, KEY_DOT},
/* 47 */    {0, KEY_SLASH},
/* 48 */    {0, KEY_0},
/* 49 */    {0, KEY_1},
/* 50 */    {0, KEY_2},
/* 51 */    {0, KEY_3},
/* 52 */    {0, KEY_4},
/* 53 */    {0, KEY_5},
/* 54 */    {0, KEY_6},
/* 55 */    {0, KEY_7},
/* 56 */    {0, KEY_8},
/* 57 */    {0, KEY_9},
/* 58 */    {MOD_SHIFT_LEFT, KEY_SEMICOLON},
/* 59 */    {0, KEY_SEMICOLON},
/* 60 */    {MOD_SHIFT_LEFT, KEY_COMMA},
/* 61 */    {0, KEY_EQ},
/* 62 */    {MOD_SHIFT_LEFT, KEY_DOT},
/* 63 */    {MOD_SHIFT_LEFT, KEY_SLASH},
/* 64 */    {MOD_SHIFT_LEFT, KEY_2},
/* 65 */    {MOD_SHIFT_LEFT, KEY_A},
/* 66 */    {MOD_SHIFT_LEFT, KEY_B},
/* 67 */    {MOD_SHIFT_LEFT, KEY_C},
/* 68 */    {MOD_SHIFT_LEFT, KEY_D},
/* 69 */    {MOD_SHIFT_LEFT, KEY_E},
/* 70 */    {MOD_SHIFT_LEFT, KEY_F},
/* 71 */    {MOD_SHIFT_LEFT, KEY_G},
/* 72 */    {MOD_SHIFT_LEFT, KEY_H},
/* 73 */    {MOD_SHIFT_LEFT, KEY_I},
/* 74 */    {MOD_SHIFT_LEFT, KEY_J},
/* 75 */    {MOD_SHIFT_LEFT, KEY_K},
/* 76 */    {MOD_SHIFT_LEFT, KEY_L},
/* 77 */    {MOD_SHIFT_LEFT, KEY_M},
/* 78 */    {MOD_SHIFT_LEFT, KEY_N},
/* 79 */    {MOD_SHIFT_LEFT, KEY_O},
/* 80 */    {MOD_SHIFT_LEFT, KEY_P},
/* 81 */    {MOD_SHIFT_LEFT, KEY_Q},
/* 82 */    {MOD_SHIFT_LEFT, KEY_R},
/* 83 */    {MOD_SHIFT_LEFT, KEY_S},
/* 84 */    {MOD_SHIFT_LEFT, KEY_T},
/* 85 */    {MOD_SHIFT_LEFT, KEY_U},
/* 86 */    {MOD_SHIFT_LEFT, KEY_V},
/* 87 */    {MOD_SHIFT_LEFT, KEY_W},
/* 88 */    {MOD_SHIFT_LEFT, KEY_X},
/* 89 */    {MOD_SHIFT_LEFT, KEY_Y},
/* 90 */    {MOD_SHIFT_LEFT, KEY_Z},
/* 91 */    {0, KEY_BRACKET_L},
/* 92 */    {0, KEY_BACKSLASH},
/* 93 */    {0, KEY_BRACKET_R},
/* 94 */    {MOD_SHIFT_LEFT, KEY_6},
/* 95 */    {MOD_SHIFT_LEFT, KEY_MINUS},
/* 96 */    {0, KEY_GRAVE_ACCENT},
/* 97 */    {0, KEY_A},
/* 98 */    {0, KEY_B},
/* 99 */    {0, KEY_C},
/* 100 */    {0, KEY_D},
/* 101 */    {0, KEY_E},
/* 102 */    {0, KEY_F},
/* 103 */    {0, KEY_G},
/* 104 */    {0, KEY_H},
/* 105 */    {0, KEY_I},
/* 106 */    {0, KEY_J},
/* 107 */    {0, KEY_K},
/* 108 */    {0, KEY_L},
/* 109 */    {0, KEY_M},
/* 110 */    {0, KEY_N},
/* 111 */    {0, KEY_O},
/* 112 */    {0, KEY_P},
/* 113 */    {0, KEY_Q},
/* 114 */    {0, KEY_R},
/* 115 */    {0, KEY_S},
/* 116 */    {0, KEY_T},
/* 117 */    {0, KEY_U},
/* 118 */    {0, KEY_V},
/* 119 */    {0, KEY_W},
/* 120 */    {0, KEY_X},
/* 121 */    {0, KEY_Y},
/* 122 */    {0, KEY_Z},
/* 123 */    {MOD_SHIFT_LEFT, KEY_BRACKET_L},
/* 124 */    {MOD_SHIFT_LEFT, KEY_BACKSLASH},
/* 125 */    {MOD_SHIFT_LEFT, KEY_BRACKET_R},
/* 126 */    {MOD_SHIFT_LEFT, KEY_GRAVE_ACCENT},
/* 127 */    {0, KEY_DEL}

};

//V-USB:
static void buildReport(uchar key)
{
/* This (not so elegant) cast saves us 10 bytes of program memory */
    *(int *)reportBuffer = pgm_read_word(keyReport[key]);
}

static const uchar  keyReportRelease[NUM_KEYS + 1][2] PROGMEM = {
/* none */  {0, 0},                     /* no key pressed */
/* 1 */    {0, KEY_},
/* 2 */    {0, KEY_},
/* 3 */    {0, KEY_},
/* 4 */    {0, KEY_},
/* 5 */    {0, KEY_},
/* 6 */    {0, KEY_},
/* 7 */    {0, KEY_},
/* 8 */    {0, KEY_},
/* 9 */    {0, KEY_TAB + 128},
/* 10 */    {0, KEY_},
/* 11 */    {0, KEY_},
/* 12 */    {0, KEY_},
/* 13 */    {0, KEY_ENTER + 128},
/* 14 */    {0, KEY_},
/* 15 */    {0, KEY_},
/* 16 */    {0, KEY_},
/* 17 */    {0, KEY_},
/* 18 */    {0, KEY_},
/* 19 */    {0, KEY_},
/* 20 */    {0, KEY_},
/* 21 */    {0, KEY_},
/* 22 */    {0, KEY_},
/* 23 */    {0, KEY_},
/* 24 */    {0, KEY_},
/* 25 */    {0, KEY_},
/* 26 */    {0, KEY_},
/* 27 */    {0, KEY_ESC + 128},
/* 28 */    {0, KEY_},
/* 29 */    {0, KEY_},
/* 30 */    {0, KEY_},
/* 31 */    {0, KEY_},
/* 32 */    {0, KEY_SPACE + 128},
/* 33 */    {MOD_SHIFT_LEFT, KEY_1 + 128},
/* 34 */    {MOD_SHIFT_LEFT, KEY_APOSTROPHE + 128},
/* 35 */    {MOD_SHIFT_LEFT, KEY_3 + 128},
/* 36 */    {MOD_SHIFT_LEFT, KEY_4 + 128},
/* 37 */    {MOD_SHIFT_LEFT, KEY_5 + 128},
/* 38 */    {MOD_SHIFT_LEFT, KEY_7 + 128},
/* 39 */    {0, KEY_APOSTROPHE + 128},
/* 40 */    {MOD_SHIFT_LEFT, KEY_9 + 128},
/* 41 */    {MOD_SHIFT_LEFT, KEY_0 + 128},
/* 42 */    {MOD_SHIFT_LEFT, KEY_8 + 128},
/* 43 */    {MOD_SHIFT_LEFT, KEY_EQ + 128},
/* 44 */    {0, KEY_COMMA + 128},
/* 45 */    {0, KEY_MINUS + 128},
/* 46 */    {0, KEY_DOT + 128},
/* 47 */    {0, KEY_SLASH + 128},
/* 48 */    {0, KEY_0 + 128},
/* 49 */    {0, KEY_1 + 128},
/* 50 */    {0, KEY_2 + 128},
/* 51 */    {0, KEY_3 + 128},
/* 52 */    {0, KEY_4 + 128},
/* 53 */    {0, KEY_5 + 128},
/* 54 */    {0, KEY_6 + 128},
/* 55 */    {0, KEY_7 + 128},
/* 56 */    {0, KEY_8 + 128},
/* 57 */    {0, KEY_9 + 128},
/* 58 */    {MOD_SHIFT_LEFT, KEY_SEMICOLON + 128},
/* 59 */    {0, KEY_SEMICOLON + 128},
/* 60 */    {MOD_SHIFT_LEFT, KEY_COMMA + 128},
/* 61 */    {0, KEY_EQ + 128},
/* 62 */    {MOD_SHIFT_LEFT, KEY_DOT + 128},
/* 63 */    {MOD_SHIFT_LEFT, KEY_SLASH + 128},
/* 64 */    {MOD_SHIFT_LEFT, KEY_2 + 128},
/* 65 */    {MOD_SHIFT_LEFT, KEY_A + 128},
/* 66 */    {MOD_SHIFT_LEFT, KEY_B + 128},
/* 67 */    {MOD_SHIFT_LEFT, KEY_C + 128},
/* 68 */    {MOD_SHIFT_LEFT, KEY_D + 128},
/* 69 */    {MOD_SHIFT_LEFT, KEY_E + 128},
/* 70 */    {MOD_SHIFT_LEFT, KEY_F + 128},
/* 71 */    {MOD_SHIFT_LEFT, KEY_G + 128},
/* 72 */    {MOD_SHIFT_LEFT, KEY_H + 128},
/* 73 */    {MOD_SHIFT_LEFT, KEY_I + 128},
/* 74 */    {MOD_SHIFT_LEFT, KEY_J + 128},
/* 75 */    {MOD_SHIFT_LEFT, KEY_K + 128},
/* 76 */    {MOD_SHIFT_LEFT, KEY_L + 128},
/* 77 */    {MOD_SHIFT_LEFT, KEY_M + 128},
/* 78 */    {MOD_SHIFT_LEFT, KEY_N + 128},
/* 79 */    {MOD_SHIFT_LEFT, KEY_O + 128},
/* 80 */    {MOD_SHIFT_LEFT, KEY_P + 128},
/* 81 */    {MOD_SHIFT_LEFT, KEY_Q + 128},
/* 82 */    {MOD_SHIFT_LEFT, KEY_R + 128},
/* 83 */    {MOD_SHIFT_LEFT, KEY_S + 128},
/* 84 */    {MOD_SHIFT_LEFT, KEY_T + 128},
/* 85 */    {MOD_SHIFT_LEFT, KEY_U + 128},
/* 86 */    {MOD_SHIFT_LEFT, KEY_V + 128},
/* 87 */    {MOD_SHIFT_LEFT, KEY_W + 128},
/* 88 */    {MOD_SHIFT_LEFT, KEY_X + 128},
/* 89 */    {MOD_SHIFT_LEFT, KEY_Y + 128},
/* 90 */    {MOD_SHIFT_LEFT, KEY_Z + 128},
/* 91 */    {0, KEY_BRACKET_L + 128},
/* 92 */    {0, KEY_BACKSLASH + 128},
/* 93 */    {0, KEY_BRACKET_R + 128},
/* 94 */    {MOD_SHIFT_LEFT, KEY_6 + 128},
/* 95 */    {MOD_SHIFT_LEFT, KEY_MINUS + 128},
/* 96 */    {0, KEY_GRAVE_ACCENT + 128},
/* 97 */    {0, KEY_A + 128},
/* 98 */    {0, KEY_B + 128},
/* 99 */    {0, KEY_C + 128},
/* 100 */    {0, KEY_D + 128},
/* 101 */    {0, KEY_E + 128},
/* 102 */    {0, KEY_F + 128},
/* 103 */    {0, KEY_G + 128},
/* 104 */    {0, KEY_H + 128},
/* 105 */    {0, KEY_I + 128},
/* 106 */    {0, KEY_J + 128},
/* 107 */    {0, KEY_K + 128},
/* 108 */    {0, KEY_L + 128},
/* 109 */    {0, KEY_M + 128},
/* 110 */    {0, KEY_N + 128},
/* 111 */    {0, KEY_O + 128},
/* 112 */    {0, KEY_P + 128},
/* 113 */    {0, KEY_Q + 128},
/* 114 */    {0, KEY_R + 128},
/* 115 */    {0, KEY_S + 128},
/* 116 */    {0, KEY_T + 128},
/* 117 */    {0, KEY_U + 128},
/* 118 */    {0, KEY_V + 128},
/* 119 */    {0, KEY_W + 128},
/* 120 */    {0, KEY_X + 128},
/* 121 */    {0, KEY_Y + 128},
/* 122 */    {0, KEY_Z + 128},
/* 123 */    {MOD_SHIFT_LEFT, KEY_BRACKET_L + 128},
/* 124 */    {MOD_SHIFT_LEFT, KEY_BACKSLASH + 128},
/* 125 */    {MOD_SHIFT_LEFT, KEY_BRACKET_R + 128},
/* 126 */    {MOD_SHIFT_LEFT, KEY_GRAVE_ACCENT + 128},
/* 127 */    {0, KEY_DEL + 128}

};

//V-USB:
static void buildReportRelease(uchar key)
{
/* This (not so elegant) cast saves us 10 bytes of program memory */
    *(int *)reportBuffer = pgm_read_word(keyReportRelease[key]);
}

//V-USB:
uchar	usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    usbMsgPtr = reportBuffer;
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            /* we only have one report type, so don't look at wValue */
            buildReport(red_character);
            return sizeof(reportBuffer);
        }else if(rq->bRequest == USBRQ_HID_GET_IDLE){
            usbMsgPtr = &idleRate;
            return 1;
        }else if(rq->bRequest == USBRQ_HID_SET_IDLE){
            idleRate = rq->wValue.bytes[1];
        }
    }else{
        /* no vendor specific requests implemented */
    }
	return 0;
}

/* ------------------------------------------------------------------------- */

//***************************************************************************
//Function: if flag=READ then to read file from SD card and send contents to USB
//if flag=VERIFY then functions will verify whether a specified file is already existing
//Arguments: flag (READ or VERIFY) and pointer to the file name
//return: 0, if normal operation or flag is READ
//            1, if file is already existing and flag = VERIFY
//        2, if file name is incompatible
//***************************************************************************
unsigned char readAndSendFile (unsigned char flag, unsigned char *fileName)
{
struct dir_Structure *dir;
unsigned long cluster, byteCounter = 0, fileSize, firstSector;
unsigned int k,i;
unsigned char j, error;

PORTD |= (1<<6); // LED Reading SD CARD => ON

error = convertFileName (fileName); //convert fileName into FAT format
if(error) return 2;

dir = findFiles (GET_FILE, fileName); //get the file location
if(dir == 0)
{
  TX_NEWLINE;transmitString_F(PSTR("Macro cannot be sent - aborting"));TX_NEWLINE;
  PORTD &= ~(1<<6); // LED Reading SD CARD => OFF
  PORTD |= (1<<5);    //LED 'SD Card READ ERROR' => ON
  _delay_ms (1000);
  PORTD &= ~(1<<5);    //LED 'SD Card READ ERROR' => OFF
  return (0);
}

if(flag == VERIFY) return (1);      //specified file name is already existing

cluster = (((unsigned long) dir->firstClusterHI) << 16) | dir->firstClusterLO;

fileSize = dir->fileSize;
TX_NEWLINE;transmitString_F(PSTR("[ Macro transmission STARTS ]"));TX_NEWLINE;	

while(1)
{
  firstSector = getFirstSector (cluster);

  for(j=0; j<8; j++)
  {
	if ( SD_readSingleBlock(firstSector + j) != 0 ) // If reading of block fails then error is sent
	{												// to UART in infinite loop.
		while (1)
		{
			TX_NEWLINE;transmitString_F(PSTR("[ SD_readSingleBlock ERROR ]"));TX_NEWLINE;
			_delay_ms (1000);
			PORTD |= (1<<5);    //LED 'SD Card READ ERROR' => ON
			_delay_ms (1000);
			PORTD &= ~(1<<5);    //LED 'SD Card READ ERROR' => OFF
		}
	}

    for(k=0; k<512; k++)
    {

    	do usbPoll(); while (!usbInterruptIsReady()); //transmit of actual char
    	buildReport(buffer[k]);
    	usbSetInterrupt(reportBuffer, sizeof(reportBuffer));
	
		do usbPoll(); while (!usbInterruptIsReady()); //transmit of key released
		buildReportRelease(buffer[k]);
		usbSetInterrupt(reportBuffer, sizeof(reportBuffer));

        transmitByte(buffer[k]);

      if ((byteCounter++) >= fileSize || PINC == 0b11111110 ) // transmission ENDS if file ends or cancel button pressed
	  {
		     _delay_ms (200); // delay for de bouncing (if macro runs shortly)
			 PORTD &= ~(1<<6); // LED Reading SD CARD => OFF
  			 TX_NEWLINE;transmitString_F(PSTR("[ Macro transmission ENDS ]"));TX_NEWLINE;
			 return 0;
	  }
	}
  }
  cluster = getSetNextCluster (cluster, GET, 0);
  if(cluster == 0)
  {
	  transmitString_F(PSTR("Error in getting cluster"));
	  PORTD |= (1<<5);    //LED 'SD Card READ ERROR' => ON
	  return 0;
  }
}
return 0;
}



/* ------------------------------------------------------------------------- */


int	main(void)
{

//SD:
unsigned char error, FAT32_active;
unsigned int i;
unsigned char fileName[13];

_delay_ms(100);  //delay for VCC stabilization

init_devices();

while (1)
{
TX_NEWLINE;
transmitString_F (PSTR("***********************************"));TX_NEWLINE;
transmitString_F (PSTR("Testing SD card..."));TX_NEWLINE;
transmitString_F (PSTR("***********************************"));TX_NEWLINE;

PORTD |= (1<<5); //LED 'SD Card READ ERROR' => ON
				//Indicates that SD CARD is not readable yet and cannot proceed to USB.
SD_init();
SPI_HIGH_SPEED;   //SCK - 4 MHz
_delay_ms(1);

FAT32_active = 1;
error = getBootSectorData (); //read boot sector and keep necessary data in global variables
if(error)
{
	transmitString_F (PSTR("FAT32 not found!"));TX_NEWLINE;  //FAT32 incompatible drive
	FAT32_active = 0;
	//goto sd_card_reinit //in case that FAT32 was not found in last init, repeat whole operation
}
	else
		{
		PORTD |= (0<<5); //LED 'SD Card READ ERROR' => OFF
		break;
		}
}

//SD:
		transmitString_F(PSTR("File list:"));TX_NEWLINE;
		findFiles(GET_LIST,0);TX_NEWLINE;
		transmitString_F (PSTR("***********************************"));TX_NEWLINE;
//V-USB:

	transmitString_F(PSTR("USB_hardwareInit"));TX_NEWLINE;
    USB_hardwareInit();
	odDebugInit();
	usbInit();
	sei();
    DBG1(0x00, 0, 0);
	transmitString_F(PSTR("Done."));TX_NEWLINE;

		transmitString_F(PSTR("Waiting for USB initialization (after it's being plugged in)."));TX_NEWLINE;
		for(i=0; i<6; i++) //sending nothing in order to avoid missing characters in the beginning
		{                  // this will proceed when USB is plugged in - sends no key pressed
			do usbPoll(); while (!usbInterruptIsReady()); //transmit of nothing - key released
			buildReport(0);
			usbSetInterrupt(reportBuffer, sizeof(reportBuffer));
		}
		transmitString_F(PSTR("USB initialized."));TX_NEWLINE;
		PORTD |= (1<<PD7); // LED indicating "USB initialized."

	for(;;)	/* main event loop for writing of the char to USB */
	{
	
	  switch(PINA) // BUTTONS
	  {
		  case 0b11111110:
		  transmitString_F(PSTR("Button_0_pressed"));TX_NEWLINE;
		  strcpy(fileName,"macro0.txt"); //file to macro from.
		  //readFile( READ, fileName);
		  readAndSendFile (READ, fileName); //characters will be red form file and sent to USB
		  break;
		  case 0b11111101:
		  transmitString_F(PSTR("Button_1_pressed"));TX_NEWLINE;
		  strcpy(fileName,"macro1.txt"); //file to macro from.
		  //readFile( READ, fileName);
		  readAndSendFile (READ, fileName); //characters will be red form file and sent to USB
		  break;
		  case 0b11111011:
		  transmitString_F(PSTR("Button_2_pressed"));TX_NEWLINE;
		  strcpy(fileName,"macro2.txt"); //file to macro from.
		  readAndSendFile (READ, fileName); //characters will be red form file and sent to USB
		  break;
		  case 0b11110111:
		  transmitString_F(PSTR("Button_3_pressed"));TX_NEWLINE;
		  strcpy(fileName,"macro3.txt"); //file to macro from.
		  readAndSendFile (READ, fileName); //characters will be red form file and sent to USB
		  break;
		  case 0b11101111:
		  transmitString_F(PSTR("Button_4_pressed"));TX_NEWLINE;
		  strcpy(fileName,"macro4.txt"); //file to macro from.
		  readAndSendFile (READ, fileName); //characters will be red form file and sent to USB
		  break;
		  case 0b11011111:
		  transmitString_F(PSTR("Button_5_pressed"));TX_NEWLINE;
		  strcpy(fileName,"macro5.txt"); //file to macro from.
		  readAndSendFile (READ, fileName); //characters will be red form file and sent to USB
		  break;
		  case 0b10111111:
		  transmitString_F(PSTR("Button_6_pressed"));TX_NEWLINE;
		  strcpy(fileName,"macro6.txt"); //file to macro from.
		  readAndSendFile (READ, fileName); //characters will be red form file and sent to USB
		  break;
		  case 0b01111111:
		  transmitString_F(PSTR("Button_7_pressed"));TX_NEWLINE;
		  strcpy(fileName,"macro7.txt"); //file to macro from.
		  readAndSendFile (READ, fileName); //characters will be red form file and sent to USB
		  break;
		  default:
		  break;
	  }
	  
	  switch(PINC) // BUTTONS
      {
	      //case 0b11111110: - CANCEL BUTTON ON BOTTOM
	      //transmitString_F(PSTR("Button_0_pressed"));TX_NEWLINE;
		  //strcpy(fileName,"macro0.txt"); //file to macro from.
		  //readFile( READ, fileName);
		  //readAndSendFile (READ, fileName); //characters will be red form file and sent to USB
		  //break;
  	      case 0b11111101:
  	      transmitString_F(PSTR("Button_14_pressed"));TX_NEWLINE;
  	      strcpy(fileName,"macro1.txt"); //file to macro from.
		  //readFile( READ, fileName);
  	      readAndSendFile (READ, fileName); //characters will be red form file and sent to USB
  	      break;
	      case 0b11111011:
	      transmitString_F(PSTR("Button_13_pressed"));TX_NEWLINE;
	      strcpy(fileName,"macro2.txt"); //file to macro from.
	      readAndSendFile (READ, fileName); //characters will be red form file and sent to USB
	      break;
	      case 0b11110111:
	      transmitString_F(PSTR("Button_12_pressed"));TX_NEWLINE;
	      strcpy(fileName,"macro3.txt"); //file to macro from.
	      readAndSendFile (READ, fileName); //characters will be red form file and sent to USB
	      break;
	      case 0b11101111:
	      transmitString_F(PSTR("Button_11_pressed"));TX_NEWLINE;
	      strcpy(fileName,"macro4.txt"); //file to macro from.
	      readAndSendFile (READ, fileName); //characters will be red form file and sent to USB
	      break;
	      case 0b11011111:
	      transmitString_F(PSTR("Button_10_pressed"));TX_NEWLINE;
	      strcpy(fileName,"macro5.txt"); //file to macro from.
	      readAndSendFile (READ, fileName); //characters will be red form file and sent to USB
	      break;
	      case 0b10111111:
	      transmitString_F(PSTR("Button_9_pressed"));TX_NEWLINE;
	      strcpy(fileName,"macro6.txt"); //file to macro from.
	      readAndSendFile (READ, fileName); //characters will be red form file and sent to USB
	      break;
	      case 0b01111111:
	      transmitString_F(PSTR("Button_8_pressed"));TX_NEWLINE;
	      strcpy(fileName,"macro7.txt"); //file to macro from.
	      readAndSendFile (READ, fileName); //characters will be red form file and sent to USB
	      break;
		  default:
		  break;
	  }

	}
	return 0;
}

/* ------------------------------------------------------------------------- */
